from abc import ABC, abstractmethod
from typing import Iterable

from grem.models import Repository


class GitServiceProvider(ABC):
    @abstractmethod
    def get_repos(self) -> Iterable[Repository]:
        ...

    @abstractmethod
    def create_repo(self, name: str, private: bool):
        ...

    @abstractmethod
    def delete_repo(self, name: str):
        ...

    @abstractmethod
    def archive_repo(self, name: str):
        ...



import typer

from grem.providers import GitServiceProvider

repo_app = typer.Typer()


def list_repos(ctx: typer.Context):
    """
    List all remote repositories.
    """
    provider: GitServiceProvider = ctx.obj.get('provider')

    for repo in provider.get_repos():
        typer.echo(repo.full_name)


def create_repos(ctx: typer.Context, names: tuple[str], private: bool = True):
    """
    Create new repositories.
    """

    provider: GitServiceProvider = ctx.obj.get('provider')

    for name in names:
        provider.create_repo(name, private=private)


def delete_repos(ctx: typer.Context, names: tuple[str], prefix: str | None = None):
    """
    Delete existing repositories.
    """

    provider: GitServiceProvider = ctx.obj.get('provider')

    for name in names:
        provider.delete_repo(name)

    if prefix is None or typer.prompt(f'Delete ALL repositories prefixed: {prefix}?', default='n').lower() == 'n':
        ctx.exit()

    for name in map(lambda repo: repo.full_name,
                    filter(lambda repo: repo.get_prefix() == prefix, provider.get_repos())):
        provider.delete_repo(name)


def archive_repos(ctx: typer.Context, names: list[str] = typer.Argument(..., help='Names of repositories to archive')):
    """
    Archive existing repositories.
    """

    provider: GitServiceProvider = ctx.obj.get('provider')

    for name in names:
        provider.archive_repo(name)



import json
from pathlib import Path
from typing import Dict

from pydantic_settings import BaseSettings, JsonConfigSettingsSource, PydanticBaseSettingsSource

from .models import Profile

config_file = Path.home() / '.config/grem/config.json'


class Settings(BaseSettings):
    profiles: Dict[str, Profile] = {}

    def save(self):
        data = self.model_dump()
        config_file.parent.mkdir(0o755, parents=True, exist_ok=True)

        config_file.write_text(json.dumps(data, indent=4), encoding='utf-8')

    @classmethod
    def settings_customise_sources(
        cls,
        settings_cls: type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> tuple[PydanticBaseSettingsSource, ...]:
        return (JsonConfigSettingsSource(settings_cls, json_file=str(config_file)),)


settings = Settings()



from enum import Enum
from typing import Literal

from pydantic import BaseModel


class Profile(BaseModel):
    provider: Literal['github']
    access_token: str


class Repository(BaseModel):
    name: str
    full_name: str

    def get_prefix(self) -> str:
        return self.full_name[0:self.full_name.index('/')]


class ProviderType(str, Enum):
    GITHUB = 'github'



import importlib
from typing import Annotated

import typer

from grem.config import settings
from grem.models import Profile, ProviderType

app = typer.Typer()
app.command('list-repos')(commands.list_repos)
app.command('create-repos')(commands.create_repos)
app.command('delete-repos')(commands.delete_repos)
app.command('archive-repos')(commands.archive_repos)


@app.callback()
def main(
    ctx: typer.Context,
    profile: Annotated[str, typer.Option(help='Profile to use', metavar='NAME')] = 'default'
):
    ctx.ensure_object(dict)

    if profile not in settings.profiles:
        typer.echo(f'Profile "{profile}" not found, try to generate:')

        settings.profiles[profile] = Profile(
            provider=typer.prompt('Provider', type=ProviderType, default='github'),
            access_token=typer.prompt('Access token', hide_input=True)
        )

        settings.save()

    provider_module = importlib.import_module(f'gitflux.providers.{settings.profiles[profile].provider}')
    ctx.obj['provider'] = provider_module.create_provider(settings.profiles[profile].access_token)


if __name__ == '__main__':
    app()



from __future__ import annotations

from typing import Iterable

from github import Github, AuthenticatedUser, Organization, Repository as GitHubRepository

from grem.models import Repository
from grem.providers import GitServiceProvider


def create_provider(token: str) -> GitHubService:
    return GitHubService(token=token)


def convert_git_repo(github_repo: GitHubRepository) -> Repository:
    return Repository(
        name=github_repo.name,
        full_name=github_repo.full_name
    )


def parse_repo_fullname(fullname: str, user: AuthenticatedUser, orgs: Iterable[Organization]) -> tuple:
    if fullname.find('/') == -1:
        owner = user
        repo_name = fullname
    else:
        org_name, repo_name = fullname.split('/')

        if org_name == user.login:
            owner = user
        else:
            owner = next((x for x in orgs if x.login == org_name), None)

        if owner is None:
            raise NameError(f'Organization not found: {org_name}.')

    return owner, repo_name


class GitHubService(GitServiceProvider):
    api: Github
    user: AuthenticatedUser
    orgs: Iterable[Organization]

    def __init__(self, token: str):
        self.api = Github(login_or_token=token)
        self.user = self.api.get_user()
        self.orgs = self.user.get_orgs()

    def get_repos(self) -> Iterable[Repository]:
        return map(convert_git_repo, self.user.get_repos())

    def create_repo(self, name: str, private: bool):
        owner, repo_name = parse_repo_fullname(name, self.user, self.orgs)
        owner.create_repo(repo_name, private=private)

    def delete_repo(self, name: str):
        owner, repo_name = parse_repo_fullname(name, self.user, self.orgs)
        owner.get_repo(repo_name).delete()

    def archive_repo(self, name: str):
        owner, repo_name = parse_repo_fullname(name, self.user, self.orgs)
        owner.get_repo(repo_name).edit(archived=True)
